#include <MPU6050_tockn.h>
#include <Wire.h>

MPU6050 mpu6050(Wire);

long timer = 0;

#define PWM1 32
#define PWM2 33
#define PWM3 27
#define PWM4 12

int Rcode1 = 0, Lcode1 = 0;


void Rcode1callBack(void)
{
  if (digitalRead(22) == 1 )
    Rcode1++;
  else
    Rcode1--;
}

void Lcode1callBack(void)
{
  if (digitalRead(4) == 1 )
    Lcode1--;
  else
    Lcode1++;
}

void Motor_Left_Setspeed(int Motorspeed)
{
  if (Motorspeed > 0)
  {
    int PWM = Motorspeed ;  // 正转
    if (PWM > 1000)
      PWM = 1000;
    ledcWrite(0, 0);                     // 设置占空比
    ledcWrite(1, PWM);
  }
  else
  {
    int PWM = (-Motorspeed) ;  // 反转
    if (PWM > 1000)
      PWM = 1000;
    ledcWrite(0, PWM);                     // 设置占空比
    ledcWrite(1, 0);
  }
}

void Motor_Right_Setspeed(int Motorspeed)
{
  if (Motorspeed > 0)
  {
    int PWM = Motorspeed ;  // 正转
    if (PWM > 1000)
      PWM = 1000;
    ledcWrite(2, PWM);    // 设置占空比
    ledcWrite(3, 0);
  }
  else
  {
    int PWM = (-Motorspeed) ;  // 反转
    if (PWM > 1000)
      PWM = 1000;
    ledcWrite(2, 0);     // 设置占空比
    ledcWrite(3, PWM);
  }
}



int Mechanical_balance = -3;
int balance_UP_KP = 100;
int balance_UP_KD = 8;
/*******************************************************************
  函数功能：直立PD控制
  入口参数：角度、角速度
  返回  值：直立控制PWM
******************************************************************/
int balance_UP(float Angle, float Gyro)
{
  float Bias;    //  角度误差
  int balance;   //  直立环计算出来的电机转速PWM值
  Bias = Angle - Mechanical_balance;
  // 求出平衡的角度中值和机械相关
  balance = balance_UP_KP * Bias + balance_UP_KD * Gyro;
  //===计算平衡控制的电机PWM  PD控制   KP是P系数 KD是D系数
  return balance;
}

/**************************************
  函数功能：速度PI控制
  入口参数：电机编码器的值
  返回值：速度控制PWM
**************************************/
int velocity_KP = 2;
float velocity_KI = 0.01;
int delaytime = 5;
int velocity(int encoder_left, int encoder_right)
{
  static float Velocity, Encoder_Least, Encoder, Movement;
  static float Encoder_Integral;
  //=============速度PI控制器=======================//
  Encoder_Least = (encoder_left + encoder_right) - 0;    //===获取最新速度偏差    测量速度（左右编码器之和）-目标速度（此处为零）

  Encoder *= 0.7;
  Encoder += Encoder_Least * 0.3; //  低通滤波

  Encoder_Integral += Encoder;  //   积分出位移 积分时间：delaytime

  if (Encoder_Integral > 10000)    Encoder_Integral = 10000;   // 积分限幅
  if (Encoder_Integral < -10000)    Encoder_Integral = -10000; // 积分限幅

  Velocity = Encoder * velocity_KP + Encoder_Integral * velocity_KI; // 速度控制

  //  if (pitch < -40 || pitch > 40)   Encoder_Integral = 0;  //===电机关闭后清除积分

  return Velocity;
}

int Straight_KS = 20;
/**************************************
  函数功能：两侧电机同速控制
  入口参数：电机编码器的值
  返回值：电机同转速控制PWM
**************************************/
int Straight_compensation(int encoder_left, int encoder_right)
{
  int diff = encoder_left - encoder_right;
  int compensation = Straight_KS * diff;
  return compensation;
}


void setup()
{
  Serial.begin(115200);
  ledcSetup(0, 10000, 10);  //设置LEDC通道0频率为10K，分辨率为10位，即占空比可选0~1023
  ledcSetup(1, 10000, 10);
  ledcSetup(2, 10000, 10);
  ledcSetup(3, 10000, 10);

  ledcAttachPin(PWM1, 0);     //设置LEDC通道0在IO1上输出
  ledcAttachPin(PWM2, 1);
  ledcAttachPin(PWM3, 2);
  ledcAttachPin(PWM4, 3);

  ledcWrite(0, 0);        // 设置占空比
  ledcWrite(1, 0);
  ledcWrite(2, 0);
  ledcWrite(3, 0);

  pinMode(23 , INPUT_PULLDOWN); // Rcode1
  pinMode(22 , INPUT_PULLDOWN); // Rcode2
  pinMode(21 , INPUT_PULLDOWN); // Lcode1
  pinMode(4 ,  INPUT_PULLDOWN);  // Lcode2

  attachInterrupt(23, Rcode1callBack, RISING);  //当电平上升沿时，触发中断
  attachInterrupt(21, Lcode1callBack, RISING);


  Wire.begin(26, 25);
  mpu6050.begin();
  mpu6050.calcGyroOffsets(true);

}

int n = 0;
void loop()
{
  delay(delaytime);
  mpu6050.update();

  int balance_PWM = balance_UP(mpu6050.getAngleY(), mpu6050.getGyroY());
  int velocity_PWM = velocity(Lcode1, Rcode1);
  int Final_PWM = balance_PWM - velocity_PWM;
  int compensation_PWM = Straight_compensation(Lcode1, Rcode1);
  Motor_Left_Setspeed(Final_PWM);
  Motor_Right_Setspeed(Final_PWM + compensation_PWM);

  n++;
  if (n == 50)
  {
    Serial.println("===================");
    Serial.print("tgyroY : "); Serial.print(mpu6050.getGyroY());      // 打印角度
    Serial.print("\t angleY : "); Serial.println(mpu6050.getAngleY());
    Serial.print("Lcode1 = ");  Serial.print(Lcode1);                 // 打印编码器计数
    Serial.print("Rcode1 = ");  Serial.println(Rcode1);
    Serial.print("balancePWM = ");  Serial.println(balance_PWM);      // 打印直立环PWM值
    Serial.print("velocityPWM = ");  Serial.println(velocity_PWM);    // 打印速度环PWM值
    Serial.print("compensationPWM = ");  Serial.println(compensation_PWM);    // 打印同速环PWM值
    Serial.println("==================\n");
    n = 0;
  }

  Rcode1 = 0, Lcode1 = 0; // 清空计数

  if (Serial.available() > 0)//判读是否串口有数据
  {
    String comdata = "";//缓存清零
    while (Serial.available() > 0)//循环串口是否有数据
    {
      comdata += char(Serial.read()); //叠加数据到comdata
      delay(2);//延时等待
    }
    if (comdata.indexOf('p') != -1)
    {
      String substr = comdata.substring(2); // 从第三位开始截取
      balance_UP_KP = substr.toInt(); // 转成整数
    }
    else if (comdata.indexOf('d') != -1)
    {
      String substr = comdata.substring(2);  // 从第三位开始截取
      balance_UP_KD = substr.toInt(); // 转成整数
    }
    else if (comdata.indexOf('v') != -1)
    {
      String substr = comdata.substring(2);  // 从第三位开始截取
      velocity_KP = substr.toInt(); // 转成整数
    }
    else if (comdata.indexOf('i') != -1)
    {
      String substr = comdata.substring(2); // 从第三位开始截取
      velocity_KI = substr.toFloat(); // 转成整数
    }
    else if (comdata.indexOf('s') != -1)
    {
      String substr = comdata.substring(2); // 从第三位开始截取
      Straight_KS = substr.toFloat(); // 转成整数
    }
    else if (comdata.indexOf('m') != -1)
    {
      String substr = comdata.substring(2);  // 从第三位开始截取
      Mechanical_balance = substr.toInt(); // 转成整数
    }
    else if (comdata.indexOf('t') != -1)
    {
      String substr = comdata.substring(2);  // 从第三位开始截取
      delaytime = substr.toInt(); // 转成整数
    }
  }

}
